from littlefieldpy.model.Paths import Paths


class CompletedJobs:
    def __init__(self, lf):
        self.lf = lf

    def count(self, x='all'):
        return self.lf.get_data_multi(Paths.COMPLETED_JOB_COUNT, x)

    def lead_times(self, x='all'):
        return self.lf.get_data_multi(Paths.LEAD_TIMES, x)

    def revenues(self, x='all'):
        return self.lf.get_data_multi(Paths.REVENUES, x)



import re
from collections import namedtuple

from littlefieldpy.model.Paths import Paths

OrdersInfo = namedtuple('OrdersInfo',
                        ['wip_limit', 'kits_per_job', 'lot_size', 'lots_per_job', 'current_contract',
                         'quoted_lead_time', 'max_lead_time', 'revenue_per_order'])

orders_info_regex = re.compile(r'''<b>Maximum WIP Limit: </b>(\d+) jobs<BR>
<B>Number of kits in 1 job: </B>(\d+)<BR>
<B>Lot size: </B>(\d+) kits, or (\d+) lots? per job<BR>
<B>Current contract: </B>(\d+)<BR>
<DD>Quoted lead time: ([\d.]+) day\(s\)<BR>
<DD>Maximum lead time: ([\d.]+) day\(s\)<BR>
<DD>Revenue per order: ([\d.]+) dollars<BR><HR>''')


def parse_orders_info(wip_limit, kits_per_job, lot_size, lots_per_job, current_contract, quoted_lead_time,
                      max_lead_time, revenue_per_order):
    wip_limit = int(wip_limit)
    kits_per_job = int(kits_per_job)
    lot_size = int(lot_size)
    lots_per_job = int(lots_per_job)
    current_contract = int(current_contract)
    quoted_lead_time = float(quoted_lead_time)
    max_lead_time = float(max_lead_time)
    revenue_per_order = float(revenue_per_order)
    return OrdersInfo(wip_limit, kits_per_job, lot_size, lots_per_job, current_contract, quoted_lead_time,
                      max_lead_time, revenue_per_order)


class Orders:
    def __init__(self, lf):
        self.lf = lf

    def job_arrivals(self, x='all'):
        return self.lf.get_data(Paths.JOBS_IN, x)

    def queued_jobs(self, x='all'):
        return self.lf.get_data(Paths.QUEUED_JOBS, x)

    def info(self, update=False) -> OrdersInfo:
        params = {update: 'update'} if update else None
        raw = self.lf.get('OrdersMenu', params)
        m = orders_info_regex.search(raw)
        if m is None:
            self.lf.update_session_id()
            raw = self.lf.get('OrdersMenu', params)
            m = orders_info_regex.search(raw)
            if m is None:
                raise RuntimeError('failed to get customer orders')

        return parse_orders_info(*m.groups())



import os
import re
from enum import Enum
import requests

from littlefieldpy.model.CompletedJobs import CompletedJobs
from littlefieldpy.model.Materials import Materials
from littlefieldpy.model.Orders import Orders
from littlefieldpy.model.Station import Station
from littlefieldpy.model.Paths import Paths

session_id_regex = re.compile(r'''JSESSIONID=(\w+);''')
points_regex = re.compile(r"""points: '((?:\d+ [\d.]+ ?)+)'""")
multi_points_regex = re.compile(r"""label: '([\d\w\s]+)', name: '([^']+)', points: '((?:\d+ [\d.]+ ?)+)'""")


class Littlefield:
    def __init__(self, institution, team_id='', password='', base_url='https://op.responsive.net/Littlefield/'):
        self.session_id = None
        if team_id == '' or password == '':
            self.team_id, self.password, self.institution = Littlefield._get_credentials_from_environment()
        else:
            self.team_id = team_id
            self.password = password
            self.institution = institution

        if self.team_id == '' or self.password == '' or self.institution == '':
            raise ValueError('TeamID, Password, and Institution are required!')

        # Ensure URL ends with /
        self.base_url = base_url.rstrip('/') + '/'

        self.update_session_id()

        self.orders = Orders(self)
        self.materials = Materials(self)
        self.station1 = Station(self, 1)
        self.station2 = Station(self, 2)
        self.station3 = Station(self, 3)
        self.completed_jobs = CompletedJobs(self)

    def cash(self, x='all'):
        return self.get_data(Paths.CASH, x)

    @staticmethod
    def _get_credentials_from_environment():
        tid = os.getenv('LITTLEFIELD_TEAM_ID', '')
        pw = os.getenv('LITTLEFIELD_PASSWORD', '')
        institution = os.getenv('LITTLEFIELD_INSTITUTION', '')
        return tid, pw, institution

    def update_session_id(self):
        self.session_id = self._get_session_id()

    def _get_session_id(self):
        headers = {'User-Agent': ''}
        payload = {'institution': self.institution, 'ismobile': 'false', 'id': self.team_id, 'password': self.password}
        r = requests.post(self.base_url + Paths.LOGIN.value, headers=headers, data=payload)
        cookie = r.headers.get('Set-Cookie')
        m = session_id_regex.search(cookie)
        if m is None:
            raise RuntimeError(r.text)

        return m.group(1)

    def get(self, path, params=None):
        cookie = {'JSESSIONID': self.session_id}
        r = requests.get(self.base_url + path, cookies=cookie, params=params)
        return r.text

    def set(self, path, data):
        cookie = {'JSESSIONID': self.session_id}
        r = requests.post(self.base_url + path, cookies=cookie, data=data)
        return r.text

    def get_data_multi(self, data, x='all'):
        if isinstance(data, Enum):
            data = data.value

        raw = self.get('Plot?data={}&x={}'.format(data, x))
        matches = multi_points_regex.findall(raw)
        if matches is None:
            self.update_session_id()
            raw = self.get('Plot?data={}&x={}'.format(data, x))
            matches = multi_points_regex.findall(raw)
            if matches is None:
                raise RuntimeError('failed to extract data')

        data = [(lbl, name, list(self._to_points(pts))) for lbl, name, pts in matches]
        return data

    def get_data(self, data, x='all'):
        if isinstance(data, Enum):
            data = data.value

        raw = self.get('Plot?data={}&x={}'.format(data, x))
        m = points_regex.search(raw)
        if m is None:
            self.update_session_id()
            raw = self.get('Plot?data={}&x={}'.format(data, x))
            m = points_regex.search(raw)
            if m is None:
                raise RuntimeError('failed to extract data')

        data = m.group(1)
        return list(self._to_points(data))

    @staticmethod
    def _to_points(data):
        points = [float(x) for x in data.split(' ')]
        for i in range(0, len(points), 2):
            yield points[i], points[i + 1]



from enum import Enum


class Paths(Enum):
    JOBS_IN = 'JOBIN'
    QUEUED_JOBS = 'JOBQ'
    INVENTORY = 'INV'
    STATION1_QUEUE_SIZE = 'S1Q'
    STATION1_UTILISATION = 'S1UTIL'
    STATION2_QUEUE_SIZE = 'S3Q'
    STATION2_UTILISATION = 'S3UTIL'
    STATION3_QUEUE_SIZE = 'S3Q'
    STATION3_UTILISATION = 'S3UTIL'
    COMPLETED_JOB_COUNT = 'JOBOUT'
    LEAD_TIMES = 'JOBT'
    REVENUES = 'JOBREV'
    CASH = 'CASH'
    LOGIN = 'CheckAccess'



import re
from collections import namedtuple

from littlefieldpy.model.Paths import Paths

MaterialsInfo = namedtuple('MaterialsInfo',
                           ['unit_cost', 'order_cost', 'lead_time', 'reorder_point', 'order_quantity',
                            'next_arrival_quantity', 'next_arrival_eta'])

materials_info_regex = re.compile(r'''<BR><B>Unit Cost: </B> \$ ([\d.]+)
<BR><B>Order Cost: </B> \$ ([\d.]+)
<BR><B>Lead Time:</B> (\d+) day\(s\)
<BR><B>Reorder Point:</B> ([\d,]+) kits
<BR><B>Order Quantity:</B>
([\d,]+) kits
(?:<P><B>Material order of ([\d,]+)\s+kits due to arrive in ([\d.]+) simulated days)?''')


def parse_materials_info(unit_cost, order_cost, lead_time, reorder_point,
                         order_quantity, next_arrival_quantity=None, next_arrival_eta=None):
    unit_cost = float(unit_cost)
    order_cost = float(order_cost)
    lead_time = int(lead_time)
    reorder_point = int(reorder_point.replace(',', ''))
    order_quantity = int(order_quantity.replace(',', ''))
    if next_arrival_quantity is not None:
        next_arrival_quantity = int(next_arrival_quantity.replace(',', ''))
    if next_arrival_eta is not None:
        next_arrival_eta = float(next_arrival_eta)

    return MaterialsInfo(unit_cost, order_cost, lead_time, reorder_point,
                         order_quantity, next_arrival_quantity, next_arrival_eta)


class Materials:
    def __init__(self, lf):
        self.lf = lf

    def inventory(self, x='all'):
        return self.lf.get_data(Paths.INVENTORY, x)

    def info(self, update=False) -> MaterialsInfo:
        params = {update: 'update'} if update else None
        raw = self.lf.get('MaterialMenu', params)
        m = materials_info_regex.search(raw)
        if m is None:
            self.lf.update_session_id()
            raw = self.lf.get('MaterialMenu', params)
            m = materials_info_regex.search(raw)
            if m is None:
                raise RuntimeError('failed to get material info')

        return parse_materials_info(*m.groups())



from setuptools import setup, find_packages

classifiers = [
  'Development Status :: 4 - Beta',
  'Intended Audience :: Education',
  'Topic :: Education',
  'Operating System :: Microsoft :: Windows',
  'Operating System :: MacOS',
  'Operating System :: POSIX :: Linux',
  'License :: OSI Approved :: MIT License',
  'Programming Language :: Python :: 3'
]

setup(name='littlefieldpy',
      version='0.0.2',
      description='API for the Littlefield simulation',
      long_description=open('README.md').read() + '\n\n',
      url='http://github.com/AndrewKahr/littlefieldpy',
      keywords='littlefield',
      classifiers=classifiers,
      author='Andrew Kahr',
      license='MIT',
      packages=find_packages(),
      install_requires=[
          'requests',
      ],
      zip_safe=False)



import re
from collections import namedtuple

station_info_regex = re.compile(r'''<P><B> Number of Machines: </B>(\d+)<BR>
<B>Scheduling Policy: </B>(\w+)<BR>
<B>Purchase Price: </B>\$\s*([\d,]+\.\d{2})<BR>
<B>Retirement Price: </B>\$\s*([\d,]+\.\d{2})<BR>''')

StationInfo = namedtuple('StationInfo', ['num_machines', 'scheduling_policy', 'purchase_price', 'retirement_price'])


def parse_station_info(num_machines, scheduling_policy, purchase_price, retirement_price):
    num_machines = int(num_machines)
    purchase_price = float(purchase_price.replace(',', ''))
    retirement_price = float(retirement_price.replace(',', ''))

    return StationInfo(num_machines, scheduling_policy, purchase_price, retirement_price)


class Station:
    def __init__(self, lf, station_id):
        self.lf = lf
        self.station_id = station_id

    def queue_size(self, x='all'):
        return self.lf.get_data('S{}Q'.format(self.station_id), x)

    def utilization(self, x='all'):
        return self.lf.get_data('S{}UTIL'.format(self.station_id), x)

    def info(self, update=False) -> StationInfo:
        params = {update: 'update'} if update else None
        raw = self.lf.get('StationMenu?id={}'.format(self.station_id), params)
        m = station_info_regex.search(raw)
        if m is None:
            self.lf.update_session_id()
            raw = self.lf.get('StationMenu?id={}'.format(self.station_id), params)
            m = station_info_regex.search(raw)
            if m is None:
                raise RuntimeError('failed to get station info')

        return parse_station_info(*m.groups())
